# coding: utf-8

"""
    PowerBot - Webservice for algotrading

    # TERMS AND CONDITIONS The PowerBot system provides B2B services for trading at intraday power markets. By using the PowerBot service, each user agrees to the terms and conditions of this licence: 1. The user confirms that they are familiar with the exchanges trading system and all relevant rules, is professionally qualified and in possession of a trading license for the relevant exchange. 2. The user will comply with the exchanges market rules (e.g. [EPEX Spot Market Rules](https://www.epexspot.com/en/downloads#rules-fees-processes) or [Nord Pool Market Rules](https://www.nordpoolgroup.com/trading/Rules-and-regulations/)) and will not endanger the exchange system at any time with heavy load from trading algorithms or by other use. 3. The user is aware of limits imposed by the exchange. 4. The user is solely liable for actions resulting from the use of PowerBot.   # INTRODUCTION PowerBot is a web-based software service enabling algorithmic trading on intraday power exchanges such as EPEX, Nord Pool, HUPX, BSP Southpool, TGE, OPCOM or ETPA. The service is straightforward to integrate in an existing software environment and provides a variety of programming interfaces for development of individual trading algorithms and software tools. Besides enabling fully automated intraday trading, it can be used to create tools for human traders providing relevant information and trading opportunities or can be integrated in existing software tools. For further details see https://www.powerbot-trading.com  ## Knowledge Base In addition to this API guide, please find the documentation at https://docs.powerbot-trading.com - the password will be provided by the PowerBot team. If not, please reach out to us at support@powerbot-trading.com  ## Endpoints The PowerBot service is available at the following REST endpoints:  | Instance      | Base URL for REST Endpoints                                      | |---------------|------------------------------------------------------------------| | EPEX          | https://staging.powerbot-trading.com/playground/epex/v2/api      | | Nord Pool     | https://staging.powerbot-trading.com/playground/nordpool/v2/api  | | HUPX          | https://staging.powerbot-trading.com/playground/hupx/v2/api      | | BSP Southpool | https://staging.powerbot-trading.com/playground/southpool/v2/api | | TGE           | https://staging.powerbot-trading.com/playground/tge/v2/api       | | IBEX          | https://staging.powerbot-trading.com/playground/ibex/v2/api      | | CROPEX        | https://staging.powerbot-trading.com/playground/cropex/v2/api    | | OPCOM         | https://staging.powerbot-trading.com/playground/opcom/v2/api     | | ETPA          | https://staging.powerbot-trading.com/playground/etpa/v2/api      | | BRM           | https://staging.powerbot-trading.com/playground/brm/v2/api       |  Access to endpoints is secured via an API Key, which needs to be passed as an \"api_key\" header in each request.   Notes on API Keys:  * API keys are specific to Test, Staging or Production.  * API keys are generated by the system administrator and need to be requested.  ## How to generate API clients (libraries) This OpenAPI specification can be used to generate API clients (programming libraries) for a wide range of programming languages using tools like [OpenAPI Generator](https://openapi-generator.tech/). A detailed guide can be found in the [knowledge base](https://docs.powerbot-trading.com/articles/getting-started/generating-clients/).  ## PowerBot Python client For Python, a ready-made client is also available on PyPI and can be downloaded locally via:  ```shell   pip install powerbot-client ```  ## Errors The API uses standard HTTP status codes to indicate the success or failure of the API call. The body of the response will be in JSON format as follows:  ``` {   \"message\": \"... an error message ...\" } ```  ## Paging The API uses offset and limit parameters for paged operations. An X-Total-Count header is added to responses to indicate the total number of items in a paged response.  ## API Rate Limiting The API limits the number of concurrent calls to 50 - when that limit is reached, the client will receive 503 http status codes (service unavailable) with the following text:  ``` {   \"message\": \"API rate limit exceeded\" } ``` Clients should ensure that they stay within the limit for concurrent API calls.    ## Additional code samples Additional information and code samples demonstrating the use of the API can be found at in our [knowledge base](https://docs.powerbot-trading.com/docs/programmatic-access/)  # noqa: E501

    The version of the OpenAPI document: 2.21.0-rc1
    Contact: office@powerbot-trading.com
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from powerbot_asyncio_client.configuration import Configuration


class InternalTrade(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'internal_trade_id': 'str',
        'exchange': 'str',
        'exec_time': 'datetime',
        'api_timestamp': 'datetime',
        'buy_order_id': 'str',
        'buy_cl_order_id': 'str',
        'buy_txt': 'str',
        'buy_aggressor_indicator': 'AggressorIndicator',
        'buy_portfolio_id': 'str',
        'buy_delivery_area': 'str',
        'sell_order_id': 'str',
        'sell_cl_order_id': 'str',
        'sell_txt': 'str',
        'sell_aggressor_indicator': 'AggressorIndicator',
        'sell_portfolio_id': 'str',
        'sell_delivery_area': 'str',
        'contract_id': 'str',
        'contract_name': 'str',
        'delivery_start': 'datetime',
        'delivery_end': 'datetime',
        'price': 'float',
        'quantity': 'float',
        'prod': 'str',
        'buy_strategy_id': 'str',
        'sell_strategy_id': 'str'
    }

    attribute_map = {
        'internal_trade_id': 'internal_trade_id',
        'exchange': 'exchange',
        'exec_time': 'exec_time',
        'api_timestamp': 'api_timestamp',
        'buy_order_id': 'buy_order_id',
        'buy_cl_order_id': 'buy_clOrderId',
        'buy_txt': 'buy_txt',
        'buy_aggressor_indicator': 'buy_aggressor_indicator',
        'buy_portfolio_id': 'buy_portfolio_id',
        'buy_delivery_area': 'buy_delivery_area',
        'sell_order_id': 'sell_order_id',
        'sell_cl_order_id': 'sell_clOrderId',
        'sell_txt': 'sell_txt',
        'sell_aggressor_indicator': 'sell_aggressor_indicator',
        'sell_portfolio_id': 'sell_portfolio_id',
        'sell_delivery_area': 'sell_delivery_area',
        'contract_id': 'contract_id',
        'contract_name': 'contract_name',
        'delivery_start': 'delivery_start',
        'delivery_end': 'delivery_end',
        'price': 'price',
        'quantity': 'quantity',
        'prod': 'prod',
        'buy_strategy_id': 'buy_strategy_id',
        'sell_strategy_id': 'sell_strategy_id'
    }

    def __init__(self, internal_trade_id=None, exchange=None, exec_time=None, api_timestamp=None, buy_order_id=None, buy_cl_order_id=None, buy_txt=None, buy_aggressor_indicator=None, buy_portfolio_id=None, buy_delivery_area=None, sell_order_id=None, sell_cl_order_id=None, sell_txt=None, sell_aggressor_indicator=None, sell_portfolio_id=None, sell_delivery_area=None, contract_id=None, contract_name=None, delivery_start=None, delivery_end=None, price=None, quantity=None, prod=None, buy_strategy_id=None, sell_strategy_id=None, local_vars_configuration=None):  # noqa: E501
        """InternalTrade - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._internal_trade_id = None
        self._exchange = None
        self._exec_time = None
        self._api_timestamp = None
        self._buy_order_id = None
        self._buy_cl_order_id = None
        self._buy_txt = None
        self._buy_aggressor_indicator = None
        self._buy_portfolio_id = None
        self._buy_delivery_area = None
        self._sell_order_id = None
        self._sell_cl_order_id = None
        self._sell_txt = None
        self._sell_aggressor_indicator = None
        self._sell_portfolio_id = None
        self._sell_delivery_area = None
        self._contract_id = None
        self._contract_name = None
        self._delivery_start = None
        self._delivery_end = None
        self._price = None
        self._quantity = None
        self._prod = None
        self._buy_strategy_id = None
        self._sell_strategy_id = None
        self.discriminator = None

        if internal_trade_id is not None:
            self.internal_trade_id = internal_trade_id
        if exchange is not None:
            self.exchange = exchange
        if exec_time is not None:
            self.exec_time = exec_time
        if api_timestamp is not None:
            self.api_timestamp = api_timestamp
        if buy_order_id is not None:
            self.buy_order_id = buy_order_id
        if buy_cl_order_id is not None:
            self.buy_cl_order_id = buy_cl_order_id
        if buy_txt is not None:
            self.buy_txt = buy_txt
        if buy_aggressor_indicator is not None:
            self.buy_aggressor_indicator = buy_aggressor_indicator
        if buy_portfolio_id is not None:
            self.buy_portfolio_id = buy_portfolio_id
        if buy_delivery_area is not None:
            self.buy_delivery_area = buy_delivery_area
        if sell_order_id is not None:
            self.sell_order_id = sell_order_id
        if sell_cl_order_id is not None:
            self.sell_cl_order_id = sell_cl_order_id
        if sell_txt is not None:
            self.sell_txt = sell_txt
        if sell_aggressor_indicator is not None:
            self.sell_aggressor_indicator = sell_aggressor_indicator
        if sell_portfolio_id is not None:
            self.sell_portfolio_id = sell_portfolio_id
        if sell_delivery_area is not None:
            self.sell_delivery_area = sell_delivery_area
        if contract_id is not None:
            self.contract_id = contract_id
        if contract_name is not None:
            self.contract_name = contract_name
        if delivery_start is not None:
            self.delivery_start = delivery_start
        if delivery_end is not None:
            self.delivery_end = delivery_end
        if price is not None:
            self.price = price
        if quantity is not None:
            self.quantity = quantity
        if prod is not None:
            self.prod = prod
        if buy_strategy_id is not None:
            self.buy_strategy_id = buy_strategy_id
        if sell_strategy_id is not None:
            self.sell_strategy_id = sell_strategy_id

    @property
    def internal_trade_id(self):
        """Gets the internal_trade_id of this InternalTrade.  # noqa: E501


        :return: The internal_trade_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._internal_trade_id

    @internal_trade_id.setter
    def internal_trade_id(self, internal_trade_id):
        """Sets the internal_trade_id of this InternalTrade.


        :param internal_trade_id: The internal_trade_id of this InternalTrade.  # noqa: E501
        :type internal_trade_id: str
        """

        self._internal_trade_id = internal_trade_id

    @property
    def exchange(self):
        """Gets the exchange of this InternalTrade.  # noqa: E501

        The exchange against which the internal trade was executed.  # noqa: E501

        :return: The exchange of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._exchange

    @exchange.setter
    def exchange(self, exchange):
        """Sets the exchange of this InternalTrade.

        The exchange against which the internal trade was executed.  # noqa: E501

        :param exchange: The exchange of this InternalTrade.  # noqa: E501
        :type exchange: str
        """

        self._exchange = exchange

    @property
    def exec_time(self):
        """Gets the exec_time of this InternalTrade.  # noqa: E501

        The timestamp when the trade was executed (UTC time zone)  # noqa: E501

        :return: The exec_time of this InternalTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._exec_time

    @exec_time.setter
    def exec_time(self, exec_time):
        """Sets the exec_time of this InternalTrade.

        The timestamp when the trade was executed (UTC time zone)  # noqa: E501

        :param exec_time: The exec_time of this InternalTrade.  # noqa: E501
        :type exec_time: datetime
        """

        self._exec_time = exec_time

    @property
    def api_timestamp(self):
        """Gets the api_timestamp of this InternalTrade.  # noqa: E501


        :return: The api_timestamp of this InternalTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._api_timestamp

    @api_timestamp.setter
    def api_timestamp(self, api_timestamp):
        """Sets the api_timestamp of this InternalTrade.


        :param api_timestamp: The api_timestamp of this InternalTrade.  # noqa: E501
        :type api_timestamp: datetime
        """

        self._api_timestamp = api_timestamp

    @property
    def buy_order_id(self):
        """Gets the buy_order_id of this InternalTrade.  # noqa: E501

        the order_id of the underlying buy-order. If the internal trade was matched with a new order, the id remains empty.  # noqa: E501

        :return: The buy_order_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._buy_order_id

    @buy_order_id.setter
    def buy_order_id(self, buy_order_id):
        """Sets the buy_order_id of this InternalTrade.

        the order_id of the underlying buy-order. If the internal trade was matched with a new order, the id remains empty.  # noqa: E501

        :param buy_order_id: The buy_order_id of this InternalTrade.  # noqa: E501
        :type buy_order_id: str
        """

        self._buy_order_id = buy_order_id

    @property
    def buy_cl_order_id(self):
        """Gets the buy_cl_order_id of this InternalTrade.  # noqa: E501

        the client order id of the buy order  # noqa: E501

        :return: The buy_cl_order_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._buy_cl_order_id

    @buy_cl_order_id.setter
    def buy_cl_order_id(self, buy_cl_order_id):
        """Sets the buy_cl_order_id of this InternalTrade.

        the client order id of the buy order  # noqa: E501

        :param buy_cl_order_id: The buy_cl_order_id of this InternalTrade.  # noqa: E501
        :type buy_cl_order_id: str
        """

        self._buy_cl_order_id = buy_cl_order_id

    @property
    def buy_txt(self):
        """Gets the buy_txt of this InternalTrade.  # noqa: E501

        the custom text of the buy order  # noqa: E501

        :return: The buy_txt of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._buy_txt

    @buy_txt.setter
    def buy_txt(self, buy_txt):
        """Sets the buy_txt of this InternalTrade.

        the custom text of the buy order  # noqa: E501

        :param buy_txt: The buy_txt of this InternalTrade.  # noqa: E501
        :type buy_txt: str
        """

        self._buy_txt = buy_txt

    @property
    def buy_aggressor_indicator(self):
        """Gets the buy_aggressor_indicator of this InternalTrade.  # noqa: E501


        :return: The buy_aggressor_indicator of this InternalTrade.  # noqa: E501
        :rtype: AggressorIndicator
        """
        return self._buy_aggressor_indicator

    @buy_aggressor_indicator.setter
    def buy_aggressor_indicator(self, buy_aggressor_indicator):
        """Sets the buy_aggressor_indicator of this InternalTrade.


        :param buy_aggressor_indicator: The buy_aggressor_indicator of this InternalTrade.  # noqa: E501
        :type buy_aggressor_indicator: AggressorIndicator
        """

        self._buy_aggressor_indicator = buy_aggressor_indicator

    @property
    def buy_portfolio_id(self):
        """Gets the buy_portfolio_id of this InternalTrade.  # noqa: E501


        :return: The buy_portfolio_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._buy_portfolio_id

    @buy_portfolio_id.setter
    def buy_portfolio_id(self, buy_portfolio_id):
        """Sets the buy_portfolio_id of this InternalTrade.


        :param buy_portfolio_id: The buy_portfolio_id of this InternalTrade.  # noqa: E501
        :type buy_portfolio_id: str
        """

        self._buy_portfolio_id = buy_portfolio_id

    @property
    def buy_delivery_area(self):
        """Gets the buy_delivery_area of this InternalTrade.  # noqa: E501


        :return: The buy_delivery_area of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._buy_delivery_area

    @buy_delivery_area.setter
    def buy_delivery_area(self, buy_delivery_area):
        """Sets the buy_delivery_area of this InternalTrade.


        :param buy_delivery_area: The buy_delivery_area of this InternalTrade.  # noqa: E501
        :type buy_delivery_area: str
        """

        self._buy_delivery_area = buy_delivery_area

    @property
    def sell_order_id(self):
        """Gets the sell_order_id of this InternalTrade.  # noqa: E501

        The seller's order id. If the internal trade was matched with a new order, the id remains empty.  # noqa: E501

        :return: The sell_order_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._sell_order_id

    @sell_order_id.setter
    def sell_order_id(self, sell_order_id):
        """Sets the sell_order_id of this InternalTrade.

        The seller's order id. If the internal trade was matched with a new order, the id remains empty.  # noqa: E501

        :param sell_order_id: The sell_order_id of this InternalTrade.  # noqa: E501
        :type sell_order_id: str
        """

        self._sell_order_id = sell_order_id

    @property
    def sell_cl_order_id(self):
        """Gets the sell_cl_order_id of this InternalTrade.  # noqa: E501

        The sell order's client id  # noqa: E501

        :return: The sell_cl_order_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._sell_cl_order_id

    @sell_cl_order_id.setter
    def sell_cl_order_id(self, sell_cl_order_id):
        """Sets the sell_cl_order_id of this InternalTrade.

        The sell order's client id  # noqa: E501

        :param sell_cl_order_id: The sell_cl_order_id of this InternalTrade.  # noqa: E501
        :type sell_cl_order_id: str
        """

        self._sell_cl_order_id = sell_cl_order_id

    @property
    def sell_txt(self):
        """Gets the sell_txt of this InternalTrade.  # noqa: E501

        The sell order's custom text  # noqa: E501

        :return: The sell_txt of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._sell_txt

    @sell_txt.setter
    def sell_txt(self, sell_txt):
        """Sets the sell_txt of this InternalTrade.

        The sell order's custom text  # noqa: E501

        :param sell_txt: The sell_txt of this InternalTrade.  # noqa: E501
        :type sell_txt: str
        """

        self._sell_txt = sell_txt

    @property
    def sell_aggressor_indicator(self):
        """Gets the sell_aggressor_indicator of this InternalTrade.  # noqa: E501


        :return: The sell_aggressor_indicator of this InternalTrade.  # noqa: E501
        :rtype: AggressorIndicator
        """
        return self._sell_aggressor_indicator

    @sell_aggressor_indicator.setter
    def sell_aggressor_indicator(self, sell_aggressor_indicator):
        """Sets the sell_aggressor_indicator of this InternalTrade.


        :param sell_aggressor_indicator: The sell_aggressor_indicator of this InternalTrade.  # noqa: E501
        :type sell_aggressor_indicator: AggressorIndicator
        """

        self._sell_aggressor_indicator = sell_aggressor_indicator

    @property
    def sell_portfolio_id(self):
        """Gets the sell_portfolio_id of this InternalTrade.  # noqa: E501


        :return: The sell_portfolio_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._sell_portfolio_id

    @sell_portfolio_id.setter
    def sell_portfolio_id(self, sell_portfolio_id):
        """Sets the sell_portfolio_id of this InternalTrade.


        :param sell_portfolio_id: The sell_portfolio_id of this InternalTrade.  # noqa: E501
        :type sell_portfolio_id: str
        """

        self._sell_portfolio_id = sell_portfolio_id

    @property
    def sell_delivery_area(self):
        """Gets the sell_delivery_area of this InternalTrade.  # noqa: E501


        :return: The sell_delivery_area of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._sell_delivery_area

    @sell_delivery_area.setter
    def sell_delivery_area(self, sell_delivery_area):
        """Sets the sell_delivery_area of this InternalTrade.


        :param sell_delivery_area: The sell_delivery_area of this InternalTrade.  # noqa: E501
        :type sell_delivery_area: str
        """

        self._sell_delivery_area = sell_delivery_area

    @property
    def contract_id(self):
        """Gets the contract_id of this InternalTrade.  # noqa: E501

        The contract_id against which the trade was executed  # noqa: E501

        :return: The contract_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._contract_id

    @contract_id.setter
    def contract_id(self, contract_id):
        """Sets the contract_id of this InternalTrade.

        The contract_id against which the trade was executed  # noqa: E501

        :param contract_id: The contract_id of this InternalTrade.  # noqa: E501
        :type contract_id: str
        """

        self._contract_id = contract_id

    @property
    def contract_name(self):
        """Gets the contract_name of this InternalTrade.  # noqa: E501

        The contract's name against which the trade was executed.  # noqa: E501

        :return: The contract_name of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._contract_name

    @contract_name.setter
    def contract_name(self, contract_name):
        """Sets the contract_name of this InternalTrade.

        The contract's name against which the trade was executed.  # noqa: E501

        :param contract_name: The contract_name of this InternalTrade.  # noqa: E501
        :type contract_name: str
        """

        self._contract_name = contract_name

    @property
    def delivery_start(self):
        """Gets the delivery_start of this InternalTrade.  # noqa: E501

        Time when the delivery of this trade starts (UTC timezone)  # noqa: E501

        :return: The delivery_start of this InternalTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._delivery_start

    @delivery_start.setter
    def delivery_start(self, delivery_start):
        """Sets the delivery_start of this InternalTrade.

        Time when the delivery of this trade starts (UTC timezone)  # noqa: E501

        :param delivery_start: The delivery_start of this InternalTrade.  # noqa: E501
        :type delivery_start: datetime
        """

        self._delivery_start = delivery_start

    @property
    def delivery_end(self):
        """Gets the delivery_end of this InternalTrade.  # noqa: E501

        Time when the delivery of this trade ends (UTC timezone)  # noqa: E501

        :return: The delivery_end of this InternalTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._delivery_end

    @delivery_end.setter
    def delivery_end(self, delivery_end):
        """Sets the delivery_end of this InternalTrade.

        Time when the delivery of this trade ends (UTC timezone)  # noqa: E501

        :param delivery_end: The delivery_end of this InternalTrade.  # noqa: E501
        :type delivery_end: datetime
        """

        self._delivery_end = delivery_end

    @property
    def price(self):
        """Gets the price of this InternalTrade.  # noqa: E501

        Price of the trade  # noqa: E501

        :return: The price of this InternalTrade.  # noqa: E501
        :rtype: float
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this InternalTrade.

        Price of the trade  # noqa: E501

        :param price: The price of this InternalTrade.  # noqa: E501
        :type price: float
        """

        self._price = price

    @property
    def quantity(self):
        """Gets the quantity of this InternalTrade.  # noqa: E501

        Quantity in MW of the trade  # noqa: E501

        :return: The quantity of this InternalTrade.  # noqa: E501
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this InternalTrade.

        Quantity in MW of the trade  # noqa: E501

        :param quantity: The quantity of this InternalTrade.  # noqa: E501
        :type quantity: float
        """

        self._quantity = quantity

    @property
    def prod(self):
        """Gets the prod of this InternalTrade.  # noqa: E501

        Product ID  # noqa: E501

        :return: The prod of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._prod

    @prod.setter
    def prod(self, prod):
        """Sets the prod of this InternalTrade.

        Product ID  # noqa: E501

        :param prod: The prod of this InternalTrade.  # noqa: E501
        :type prod: str
        """

        self._prod = prod

    @property
    def buy_strategy_id(self):
        """Gets the buy_strategy_id of this InternalTrade.  # noqa: E501

        The strategy ID of the trade's buy order  # noqa: E501

        :return: The buy_strategy_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._buy_strategy_id

    @buy_strategy_id.setter
    def buy_strategy_id(self, buy_strategy_id):
        """Sets the buy_strategy_id of this InternalTrade.

        The strategy ID of the trade's buy order  # noqa: E501

        :param buy_strategy_id: The buy_strategy_id of this InternalTrade.  # noqa: E501
        :type buy_strategy_id: str
        """

        self._buy_strategy_id = buy_strategy_id

    @property
    def sell_strategy_id(self):
        """Gets the sell_strategy_id of this InternalTrade.  # noqa: E501

        The strategy ID of the trade's sell order  # noqa: E501

        :return: The sell_strategy_id of this InternalTrade.  # noqa: E501
        :rtype: str
        """
        return self._sell_strategy_id

    @sell_strategy_id.setter
    def sell_strategy_id(self, sell_strategy_id):
        """Sets the sell_strategy_id of this InternalTrade.

        The strategy ID of the trade's sell order  # noqa: E501

        :param sell_strategy_id: The sell_strategy_id of this InternalTrade.  # noqa: E501
        :type sell_strategy_id: str
        """

        self._sell_strategy_id = sell_strategy_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InternalTrade):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, InternalTrade):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    PowerBot - Webservice for algotrading

    # TERMS AND CONDITIONS The PowerBot system provides B2B services for trading at intraday power markets. By using the PowerBot service, each user agrees to the terms and conditions of this licence: 1. The user confirms that they are familiar with the exchanges trading system and all relevant rules, is professionally qualified and in possession of a trading license for the relevant exchange. 2. The user will comply with the exchanges market rules (e.g. [EPEX Spot Market Rules](https://www.epexspot.com/en/downloads#rules-fees-processes) or [Nord Pool Market Rules](https://www.nordpoolgroup.com/trading/Rules-and-regulations/)) and will not endanger the exchange system at any time with heavy load from trading algorithms or by other use. 3. The user is aware of limits imposed by the exchange. 4. The user is solely liable for actions resulting from the use of PowerBot.   # INTRODUCTION PowerBot is a web-based software service enabling algorithmic trading on intraday power exchanges such as EPEX, Nord Pool, HUPX, BSP Southpool, TGE, OPCOM or ETPA. The service is straightforward to integrate in an existing software environment and provides a variety of programming interfaces for development of individual trading algorithms and software tools. Besides enabling fully automated intraday trading, it can be used to create tools for human traders providing relevant information and trading opportunities or can be integrated in existing software tools. For further details see https://www.powerbot-trading.com  ## Knowledge Base In addition to this API guide, please find the documentation at https://docs.powerbot-trading.com - the password will be provided by the PowerBot team. If not, please reach out to us at support@powerbot-trading.com  ## Endpoints The PowerBot service is available at the following REST endpoints:  | Instance      | Base URL for REST Endpoints                                      | |---------------|------------------------------------------------------------------| | EPEX          | https://staging.powerbot-trading.com/playground/epex/v2/api      | | Nord Pool     | https://staging.powerbot-trading.com/playground/nordpool/v2/api  | | HUPX          | https://staging.powerbot-trading.com/playground/hupx/v2/api      | | BSP Southpool | https://staging.powerbot-trading.com/playground/southpool/v2/api | | TGE           | https://staging.powerbot-trading.com/playground/tge/v2/api       | | IBEX          | https://staging.powerbot-trading.com/playground/ibex/v2/api      | | CROPEX        | https://staging.powerbot-trading.com/playground/cropex/v2/api    | | OPCOM         | https://staging.powerbot-trading.com/playground/opcom/v2/api     | | ETPA          | https://staging.powerbot-trading.com/playground/etpa/v2/api      | | BRM           | https://staging.powerbot-trading.com/playground/brm/v2/api       |  Access to endpoints is secured via an API Key, which needs to be passed as an \"api_key\" header in each request.   Notes on API Keys:  * API keys are specific to Test, Staging or Production.  * API keys are generated by the system administrator and need to be requested.  ## How to generate API clients (libraries) This OpenAPI specification can be used to generate API clients (programming libraries) for a wide range of programming languages using tools like [OpenAPI Generator](https://openapi-generator.tech/). A detailed guide can be found in the [knowledge base](https://docs.powerbot-trading.com/articles/getting-started/generating-clients/).  ## PowerBot Python client For Python, a ready-made client is also available on PyPI and can be downloaded locally via:  ```shell   pip install powerbot-client ```  ## Errors The API uses standard HTTP status codes to indicate the success or failure of the API call. The body of the response will be in JSON format as follows:  ``` {   \"message\": \"... an error message ...\" } ```  ## Paging The API uses offset and limit parameters for paged operations. An X-Total-Count header is added to responses to indicate the total number of items in a paged response.  ## API Rate Limiting The API limits the number of concurrent calls to 50 - when that limit is reached, the client will receive 503 http status codes (service unavailable) with the following text:  ``` {   \"message\": \"API rate limit exceeded\" } ``` Clients should ensure that they stay within the limit for concurrent API calls.    ## Additional code samples Additional information and code samples demonstrating the use of the API can be found at in our [knowledge base](https://docs.powerbot-trading.com/docs/programmatic-access/)  # noqa: E501

    The version of the OpenAPI document: 2.21.0-rc1
    Contact: office@powerbot-trading.com
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from powerbot_asyncio_client.configuration import Configuration


class AuctionTrade(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'auction_id': 'str',
        'contract_id': 'str',
        'portfolio_id': 'str',
        'delivery_start': 'datetime',
        'delivery_end': 'datetime',
        'last_updated': 'datetime',
        'order_id': 'str',
        'delivery_area': 'str',
        'price': 'float',
        'currency': 'str',
        'quantity': 'float',
        'state': 'AuctionTradeState',
        'user_id': 'str',
        'strategy_id': 'str',
        'submission_type': 'AuctionTradeSubmissionType'
    }

    attribute_map = {
        'id': 'id',
        'auction_id': 'auction_id',
        'contract_id': 'contract_id',
        'portfolio_id': 'portfolio_id',
        'delivery_start': 'delivery_start',
        'delivery_end': 'delivery_end',
        'last_updated': 'last_updated',
        'order_id': 'order_id',
        'delivery_area': 'delivery_area',
        'price': 'price',
        'currency': 'currency',
        'quantity': 'quantity',
        'state': 'state',
        'user_id': 'user_id',
        'strategy_id': 'strategy_id',
        'submission_type': 'submission_type'
    }

    def __init__(self, id=None, auction_id=None, contract_id=None, portfolio_id=None, delivery_start=None, delivery_end=None, last_updated=None, order_id=None, delivery_area=None, price=None, currency=None, quantity=None, state=None, user_id=None, strategy_id=None, submission_type=None, local_vars_configuration=None):  # noqa: E501
        """AuctionTrade - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._auction_id = None
        self._contract_id = None
        self._portfolio_id = None
        self._delivery_start = None
        self._delivery_end = None
        self._last_updated = None
        self._order_id = None
        self._delivery_area = None
        self._price = None
        self._currency = None
        self._quantity = None
        self._state = None
        self._user_id = None
        self._strategy_id = None
        self._submission_type = None
        self.discriminator = None

        self.id = id
        self.auction_id = auction_id
        self.contract_id = contract_id
        if portfolio_id is not None:
            self.portfolio_id = portfolio_id
        if delivery_start is not None:
            self.delivery_start = delivery_start
        if delivery_end is not None:
            self.delivery_end = delivery_end
        if last_updated is not None:
            self.last_updated = last_updated
        self.order_id = order_id
        self.delivery_area = delivery_area
        self.price = price
        if currency is not None:
            self.currency = currency
        self.quantity = quantity
        self.state = state
        self.user_id = user_id
        if strategy_id is not None:
            self.strategy_id = strategy_id
        if submission_type is not None:
            self.submission_type = submission_type

    @property
    def id(self):
        """Gets the id of this AuctionTrade.  # noqa: E501

        The ID of the trade, provided by the exchange.  # noqa: E501

        :return: The id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this AuctionTrade.

        The ID of the trade, provided by the exchange.  # noqa: E501

        :param id: The id of this AuctionTrade.  # noqa: E501
        :type id: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def auction_id(self):
        """Gets the auction_id of this AuctionTrade.  # noqa: E501

        The ID of the auction the trade belongs to.  # noqa: E501

        :return: The auction_id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._auction_id

    @auction_id.setter
    def auction_id(self, auction_id):
        """Sets the auction_id of this AuctionTrade.

        The ID of the auction the trade belongs to.  # noqa: E501

        :param auction_id: The auction_id of this AuctionTrade.  # noqa: E501
        :type auction_id: str
        """
        if self.local_vars_configuration.client_side_validation and auction_id is None:  # noqa: E501
            raise ValueError("Invalid value for `auction_id`, must not be `None`")  # noqa: E501

        self._auction_id = auction_id

    @property
    def contract_id(self):
        """Gets the contract_id of this AuctionTrade.  # noqa: E501

        The ID of the contract the trade belongs to.  # noqa: E501

        :return: The contract_id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._contract_id

    @contract_id.setter
    def contract_id(self, contract_id):
        """Sets the contract_id of this AuctionTrade.

        The ID of the contract the trade belongs to.  # noqa: E501

        :param contract_id: The contract_id of this AuctionTrade.  # noqa: E501
        :type contract_id: str
        """
        if self.local_vars_configuration.client_side_validation and contract_id is None:  # noqa: E501
            raise ValueError("Invalid value for `contract_id`, must not be `None`")  # noqa: E501

        self._contract_id = contract_id

    @property
    def portfolio_id(self):
        """Gets the portfolio_id of this AuctionTrade.  # noqa: E501

        The portfolio ID of the trade.  # noqa: E501

        :return: The portfolio_id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._portfolio_id

    @portfolio_id.setter
    def portfolio_id(self, portfolio_id):
        """Sets the portfolio_id of this AuctionTrade.

        The portfolio ID of the trade.  # noqa: E501

        :param portfolio_id: The portfolio_id of this AuctionTrade.  # noqa: E501
        :type portfolio_id: str
        """

        self._portfolio_id = portfolio_id

    @property
    def delivery_start(self):
        """Gets the delivery_start of this AuctionTrade.  # noqa: E501

        The delivery start of the trade.  # noqa: E501

        :return: The delivery_start of this AuctionTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._delivery_start

    @delivery_start.setter
    def delivery_start(self, delivery_start):
        """Sets the delivery_start of this AuctionTrade.

        The delivery start of the trade.  # noqa: E501

        :param delivery_start: The delivery_start of this AuctionTrade.  # noqa: E501
        :type delivery_start: datetime
        """

        self._delivery_start = delivery_start

    @property
    def delivery_end(self):
        """Gets the delivery_end of this AuctionTrade.  # noqa: E501

        The delivery end of the trade.  # noqa: E501

        :return: The delivery_end of this AuctionTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._delivery_end

    @delivery_end.setter
    def delivery_end(self, delivery_end):
        """Sets the delivery_end of this AuctionTrade.

        The delivery end of the trade.  # noqa: E501

        :param delivery_end: The delivery_end of this AuctionTrade.  # noqa: E501
        :type delivery_end: datetime
        """

        self._delivery_end = delivery_end

    @property
    def last_updated(self):
        """Gets the last_updated of this AuctionTrade.  # noqa: E501

        The timestamp of the last update of the trade.  # noqa: E501

        :return: The last_updated of this AuctionTrade.  # noqa: E501
        :rtype: datetime
        """
        return self._last_updated

    @last_updated.setter
    def last_updated(self, last_updated):
        """Sets the last_updated of this AuctionTrade.

        The timestamp of the last update of the trade.  # noqa: E501

        :param last_updated: The last_updated of this AuctionTrade.  # noqa: E501
        :type last_updated: datetime
        """

        self._last_updated = last_updated

    @property
    def order_id(self):
        """Gets the order_id of this AuctionTrade.  # noqa: E501

        The ID of the order the trade belongs to.  # noqa: E501

        :return: The order_id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._order_id

    @order_id.setter
    def order_id(self, order_id):
        """Sets the order_id of this AuctionTrade.

        The ID of the order the trade belongs to.  # noqa: E501

        :param order_id: The order_id of this AuctionTrade.  # noqa: E501
        :type order_id: str
        """
        if self.local_vars_configuration.client_side_validation and order_id is None:  # noqa: E501
            raise ValueError("Invalid value for `order_id`, must not be `None`")  # noqa: E501

        self._order_id = order_id

    @property
    def delivery_area(self):
        """Gets the delivery_area of this AuctionTrade.  # noqa: E501

        The delivery area of the trade.  # noqa: E501

        :return: The delivery_area of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._delivery_area

    @delivery_area.setter
    def delivery_area(self, delivery_area):
        """Sets the delivery_area of this AuctionTrade.

        The delivery area of the trade.  # noqa: E501

        :param delivery_area: The delivery_area of this AuctionTrade.  # noqa: E501
        :type delivery_area: str
        """
        if self.local_vars_configuration.client_side_validation and delivery_area is None:  # noqa: E501
            raise ValueError("Invalid value for `delivery_area`, must not be `None`")  # noqa: E501

        self._delivery_area = delivery_area

    @property
    def price(self):
        """Gets the price of this AuctionTrade.  # noqa: E501

        The price of the trade.  # noqa: E501

        :return: The price of this AuctionTrade.  # noqa: E501
        :rtype: float
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this AuctionTrade.

        The price of the trade.  # noqa: E501

        :param price: The price of this AuctionTrade.  # noqa: E501
        :type price: float
        """
        if self.local_vars_configuration.client_side_validation and price is None:  # noqa: E501
            raise ValueError("Invalid value for `price`, must not be `None`")  # noqa: E501

        self._price = price

    @property
    def currency(self):
        """Gets the currency of this AuctionTrade.  # noqa: E501


        :return: The currency of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this AuctionTrade.


        :param currency: The currency of this AuctionTrade.  # noqa: E501
        :type currency: str
        """

        self._currency = currency

    @property
    def quantity(self):
        """Gets the quantity of this AuctionTrade.  # noqa: E501

        The quantity of the trade.  # noqa: E501

        :return: The quantity of this AuctionTrade.  # noqa: E501
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this AuctionTrade.

        The quantity of the trade.  # noqa: E501

        :param quantity: The quantity of this AuctionTrade.  # noqa: E501
        :type quantity: float
        """
        if self.local_vars_configuration.client_side_validation and quantity is None:  # noqa: E501
            raise ValueError("Invalid value for `quantity`, must not be `None`")  # noqa: E501

        self._quantity = quantity

    @property
    def state(self):
        """Gets the state of this AuctionTrade.  # noqa: E501


        :return: The state of this AuctionTrade.  # noqa: E501
        :rtype: AuctionTradeState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this AuctionTrade.


        :param state: The state of this AuctionTrade.  # noqa: E501
        :type state: AuctionTradeState
        """
        if self.local_vars_configuration.client_side_validation and state is None:  # noqa: E501
            raise ValueError("Invalid value for `state`, must not be `None`")  # noqa: E501

        self._state = state

    @property
    def user_id(self):
        """Gets the user_id of this AuctionTrade.  # noqa: E501

        The exchange user that was used to place the order  # noqa: E501

        :return: The user_id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this AuctionTrade.

        The exchange user that was used to place the order  # noqa: E501

        :param user_id: The user_id of this AuctionTrade.  # noqa: E501
        :type user_id: str
        """
        if self.local_vars_configuration.client_side_validation and user_id is None:  # noqa: E501
            raise ValueError("Invalid value for `user_id`, must not be `None`")  # noqa: E501

        self._user_id = user_id

    @property
    def strategy_id(self):
        """Gets the strategy_id of this AuctionTrade.  # noqa: E501


        :return: The strategy_id of this AuctionTrade.  # noqa: E501
        :rtype: str
        """
        return self._strategy_id

    @strategy_id.setter
    def strategy_id(self, strategy_id):
        """Sets the strategy_id of this AuctionTrade.


        :param strategy_id: The strategy_id of this AuctionTrade.  # noqa: E501
        :type strategy_id: str
        """

        self._strategy_id = strategy_id

    @property
    def submission_type(self):
        """Gets the submission_type of this AuctionTrade.  # noqa: E501


        :return: The submission_type of this AuctionTrade.  # noqa: E501
        :rtype: AuctionTradeSubmissionType
        """
        return self._submission_type

    @submission_type.setter
    def submission_type(self, submission_type):
        """Sets the submission_type of this AuctionTrade.


        :param submission_type: The submission_type of this AuctionTrade.  # noqa: E501
        :type submission_type: AuctionTradeSubmissionType
        """

        self._submission_type = submission_type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AuctionTrade):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AuctionTrade):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    PowerBot - Webservice for algotrading

    # TERMS AND CONDITIONS The PowerBot system provides B2B services for trading at intraday power markets. By using the PowerBot service, each user agrees to the terms and conditions of this licence: 1. The user confirms that they are familiar with the exchanges trading system and all relevant rules, is professionally qualified and in possession of a trading license for the relevant exchange. 2. The user will comply with the exchanges market rules (e.g. [EPEX Spot Market Rules](https://www.epexspot.com/en/downloads#rules-fees-processes) or [Nord Pool Market Rules](https://www.nordpoolgroup.com/trading/Rules-and-regulations/)) and will not endanger the exchange system at any time with heavy load from trading algorithms or by other use. 3. The user is aware of limits imposed by the exchange. 4. The user is solely liable for actions resulting from the use of PowerBot.   # INTRODUCTION PowerBot is a web-based software service enabling algorithmic trading on intraday power exchanges such as EPEX, Nord Pool, HUPX, BSP Southpool, TGE, OPCOM or ETPA. The service is straightforward to integrate in an existing software environment and provides a variety of programming interfaces for development of individual trading algorithms and software tools. Besides enabling fully automated intraday trading, it can be used to create tools for human traders providing relevant information and trading opportunities or can be integrated in existing software tools. For further details see https://www.powerbot-trading.com  ## Knowledge Base In addition to this API guide, please find the documentation at https://docs.powerbot-trading.com - the password will be provided by the PowerBot team. If not, please reach out to us at support@powerbot-trading.com  ## Endpoints The PowerBot service is available at the following REST endpoints:  | Instance      | Base URL for REST Endpoints                                      | |---------------|------------------------------------------------------------------| | EPEX          | https://staging.powerbot-trading.com/playground/epex/v2/api      | | Nord Pool     | https://staging.powerbot-trading.com/playground/nordpool/v2/api  | | HUPX          | https://staging.powerbot-trading.com/playground/hupx/v2/api      | | BSP Southpool | https://staging.powerbot-trading.com/playground/southpool/v2/api | | TGE           | https://staging.powerbot-trading.com/playground/tge/v2/api       | | IBEX          | https://staging.powerbot-trading.com/playground/ibex/v2/api      | | CROPEX        | https://staging.powerbot-trading.com/playground/cropex/v2/api    | | OPCOM         | https://staging.powerbot-trading.com/playground/opcom/v2/api     | | ETPA          | https://staging.powerbot-trading.com/playground/etpa/v2/api      | | BRM           | https://staging.powerbot-trading.com/playground/brm/v2/api       |  Access to endpoints is secured via an API Key, which needs to be passed as an \"api_key\" header in each request.   Notes on API Keys:  * API keys are specific to Test, Staging or Production.  * API keys are generated by the system administrator and need to be requested.  ## How to generate API clients (libraries) This OpenAPI specification can be used to generate API clients (programming libraries) for a wide range of programming languages using tools like [OpenAPI Generator](https://openapi-generator.tech/). A detailed guide can be found in the [knowledge base](https://docs.powerbot-trading.com/articles/getting-started/generating-clients/).  ## PowerBot Python client For Python, a ready-made client is also available on PyPI and can be downloaded locally via:  ```shell   pip install powerbot-client ```  ## Errors The API uses standard HTTP status codes to indicate the success or failure of the API call. The body of the response will be in JSON format as follows:  ``` {   \"message\": \"... an error message ...\" } ```  ## Paging The API uses offset and limit parameters for paged operations. An X-Total-Count header is added to responses to indicate the total number of items in a paged response.  ## API Rate Limiting The API limits the number of concurrent calls to 50 - when that limit is reached, the client will receive 503 http status codes (service unavailable) with the following text:  ``` {   \"message\": \"API rate limit exceeded\" } ``` Clients should ensure that they stay within the limit for concurrent API calls.    ## Additional code samples Additional information and code samples demonstrating the use of the API can be found at in our [knowledge base](https://docs.powerbot-trading.com/docs/programmatic-access/)  # noqa: E501

    The version of the OpenAPI document: 2.21.0-rc1
    Contact: office@powerbot-trading.com
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from powerbot_asyncio_client.configuration import Configuration


class OrderModify(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'action': 'str',
        'validity_res': 'ValidityRestriction',
        'validity_date': 'datetime',
        'type': 'OrderType',
        'txt': 'str',
        'ordr_exe_restriction': 'OrderExecutionRestriction',
        'quantity': 'float',
        'display_qty': 'float',
        'price': 'float',
        'ppd': 'float',
        'expected_net_pos': 'float',
        'expected_min_net_pos': 'float',
        'expected_max_net_pos': 'float',
        'expected_min_net_pos_exposure': 'float',
        'expected_max_net_pos_exposure': 'float',
        'expected_abs_pos': 'float',
        'expected_sell_abs_pos': 'float',
        'expected_buy_abs_pos': 'float',
        'location': 'str'
    }

    attribute_map = {
        'action': 'action',
        'validity_res': 'validityRes',
        'validity_date': 'validityDate',
        'type': 'type',
        'txt': 'txt',
        'ordr_exe_restriction': 'ordrExeRestriction',
        'quantity': 'quantity',
        'display_qty': 'displayQty',
        'price': 'price',
        'ppd': 'ppd',
        'expected_net_pos': 'expectedNetPos',
        'expected_min_net_pos': 'expectedMinNetPos',
        'expected_max_net_pos': 'expectedMaxNetPos',
        'expected_min_net_pos_exposure': 'expected_min_net_pos_exposure',
        'expected_max_net_pos_exposure': 'expected_max_net_pos_exposure',
        'expected_abs_pos': 'expectedAbsPos',
        'expected_sell_abs_pos': 'expectedSellAbsPos',
        'expected_buy_abs_pos': 'expectedBuyAbsPos',
        'location': 'location'
    }

    def __init__(self, action=None, validity_res=None, validity_date=None, type=None, txt=None, ordr_exe_restriction=None, quantity=None, display_qty=None, price=None, ppd=None, expected_net_pos=None, expected_min_net_pos=None, expected_max_net_pos=None, expected_min_net_pos_exposure=None, expected_max_net_pos_exposure=None, expected_abs_pos=None, expected_sell_abs_pos=None, expected_buy_abs_pos=None, location=None, local_vars_configuration=None):  # noqa: E501
        """OrderModify - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._action = None
        self._validity_res = None
        self._validity_date = None
        self._type = None
        self._txt = None
        self._ordr_exe_restriction = None
        self._quantity = None
        self._display_qty = None
        self._price = None
        self._ppd = None
        self._expected_net_pos = None
        self._expected_min_net_pos = None
        self._expected_max_net_pos = None
        self._expected_min_net_pos_exposure = None
        self._expected_max_net_pos_exposure = None
        self._expected_abs_pos = None
        self._expected_sell_abs_pos = None
        self._expected_buy_abs_pos = None
        self._location = None
        self.discriminator = None

        self.action = action
        if validity_res is not None:
            self.validity_res = validity_res
        if validity_date is not None:
            self.validity_date = validity_date
        if type is not None:
            self.type = type
        if txt is not None:
            self.txt = txt
        if ordr_exe_restriction is not None:
            self.ordr_exe_restriction = ordr_exe_restriction
        if quantity is not None:
            self.quantity = quantity
        if display_qty is not None:
            self.display_qty = display_qty
        if price is not None:
            self.price = price
        if ppd is not None:
            self.ppd = ppd
        if expected_net_pos is not None:
            self.expected_net_pos = expected_net_pos
        if expected_min_net_pos is not None:
            self.expected_min_net_pos = expected_min_net_pos
        if expected_max_net_pos is not None:
            self.expected_max_net_pos = expected_max_net_pos
        if expected_min_net_pos_exposure is not None:
            self.expected_min_net_pos_exposure = expected_min_net_pos_exposure
        if expected_max_net_pos_exposure is not None:
            self.expected_max_net_pos_exposure = expected_max_net_pos_exposure
        if expected_abs_pos is not None:
            self.expected_abs_pos = expected_abs_pos
        if expected_sell_abs_pos is not None:
            self.expected_sell_abs_pos = expected_sell_abs_pos
        if expected_buy_abs_pos is not None:
            self.expected_buy_abs_pos = expected_buy_abs_pos
        if location is not None:
            self.location = location

    @property
    def action(self):
        """Gets the action of this OrderModify.  # noqa: E501

        * ACTI: Activates the order. Ignored if already active. * DEAC: Deactivates (hibernates) the order. Hibernated orders are removed from the order book but are still available for modification or activation in the own orders list. * MODI: Modify the order * DELE: Delete the order  # noqa: E501

        :return: The action of this OrderModify.  # noqa: E501
        :rtype: str
        """
        return self._action

    @action.setter
    def action(self, action):
        """Sets the action of this OrderModify.

        * ACTI: Activates the order. Ignored if already active. * DEAC: Deactivates (hibernates) the order. Hibernated orders are removed from the order book but are still available for modification or activation in the own orders list. * MODI: Modify the order * DELE: Delete the order  # noqa: E501

        :param action: The action of this OrderModify.  # noqa: E501
        :type action: str
        """
        if self.local_vars_configuration.client_side_validation and action is None:  # noqa: E501
            raise ValueError("Invalid value for `action`, must not be `None`")  # noqa: E501
        allowed_values = ["ACTI", "DEAC", "MODI", "DELE"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and action not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `action` ({0}), must be one of {1}"  # noqa: E501
                .format(action, allowed_values)
            )

        self._action = action

    @property
    def validity_res(self):
        """Gets the validity_res of this OrderModify.  # noqa: E501


        :return: The validity_res of this OrderModify.  # noqa: E501
        :rtype: ValidityRestriction
        """
        return self._validity_res

    @validity_res.setter
    def validity_res(self, validity_res):
        """Sets the validity_res of this OrderModify.


        :param validity_res: The validity_res of this OrderModify.  # noqa: E501
        :type validity_res: ValidityRestriction
        """

        self._validity_res = validity_res

    @property
    def validity_date(self):
        """Gets the validity_date of this OrderModify.  # noqa: E501

        Set this field, if you want to update the validity date of the order. Only mandatory in case of validityRes equals \"GTD\". It is used to define the date until which the order is valid. The remaining part of the order will be removed from the order book after this point in time.  # noqa: E501

        :return: The validity_date of this OrderModify.  # noqa: E501
        :rtype: datetime
        """
        return self._validity_date

    @validity_date.setter
    def validity_date(self, validity_date):
        """Sets the validity_date of this OrderModify.

        Set this field, if you want to update the validity date of the order. Only mandatory in case of validityRes equals \"GTD\". It is used to define the date until which the order is valid. The remaining part of the order will be removed from the order book after this point in time.  # noqa: E501

        :param validity_date: The validity_date of this OrderModify.  # noqa: E501
        :type validity_date: datetime
        """

        self._validity_date = validity_date

    @property
    def type(self):
        """Gets the type of this OrderModify.  # noqa: E501


        :return: The type of this OrderModify.  # noqa: E501
        :rtype: OrderType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OrderModify.


        :param type: The type of this OrderModify.  # noqa: E501
        :type type: OrderType
        """

        self._type = type

    @property
    def txt(self):
        """Gets the txt of this OrderModify.  # noqa: E501

        Set this field if you want to update the order's 'custom text'  # noqa: E501

        :return: The txt of this OrderModify.  # noqa: E501
        :rtype: str
        """
        return self._txt

    @txt.setter
    def txt(self, txt):
        """Sets the txt of this OrderModify.

        Set this field if you want to update the order's 'custom text'  # noqa: E501

        :param txt: The txt of this OrderModify.  # noqa: E501
        :type txt: str
        """

        self._txt = txt

    @property
    def ordr_exe_restriction(self):
        """Gets the ordr_exe_restriction of this OrderModify.  # noqa: E501


        :return: The ordr_exe_restriction of this OrderModify.  # noqa: E501
        :rtype: OrderExecutionRestriction
        """
        return self._ordr_exe_restriction

    @ordr_exe_restriction.setter
    def ordr_exe_restriction(self, ordr_exe_restriction):
        """Sets the ordr_exe_restriction of this OrderModify.


        :param ordr_exe_restriction: The ordr_exe_restriction of this OrderModify.  # noqa: E501
        :type ordr_exe_restriction: OrderExecutionRestriction
        """

        self._ordr_exe_restriction = ordr_exe_restriction

    @property
    def quantity(self):
        """Gets the quantity of this OrderModify.  # noqa: E501

        Set this field if you want to update the order's quantity.  # noqa: E501

        :return: The quantity of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this OrderModify.

        Set this field if you want to update the order's quantity.  # noqa: E501

        :param quantity: The quantity of this OrderModify.  # noqa: E501
        :type quantity: float
        """

        self._quantity = quantity

    @property
    def display_qty(self):
        """Gets the display_qty of this OrderModify.  # noqa: E501

        Set this field if you want to update the display quantity of an iceberg Order. This field is required only in the case of type=\"I\".  # noqa: E501

        :return: The display_qty of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._display_qty

    @display_qty.setter
    def display_qty(self, display_qty):
        """Sets the display_qty of this OrderModify.

        Set this field if you want to update the display quantity of an iceberg Order. This field is required only in the case of type=\"I\".  # noqa: E501

        :param display_qty: The display_qty of this OrderModify.  # noqa: E501
        :type display_qty: float
        """

        self._display_qty = display_qty

    @property
    def price(self):
        """Gets the price of this OrderModify.  # noqa: E501

        Set this field if you want to update the order's description.  # noqa: E501

        :return: The price of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this OrderModify.

        Set this field if you want to update the order's description.  # noqa: E501

        :param price: The price of this OrderModify.  # noqa: E501
        :type price: float
        """

        self._price = price

    @property
    def ppd(self):
        """Gets the ppd of this OrderModify.  # noqa: E501

        Set this field if you want to update the Peak price delta for iceberg orders. * The ppd of buy orders must be smaller or equal than zero. * The ppd of sell orders must be greater or equal than zero. If it is omitted the system will assume a value of `0.00`.  # noqa: E501

        :return: The ppd of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._ppd

    @ppd.setter
    def ppd(self, ppd):
        """Sets the ppd of this OrderModify.

        Set this field if you want to update the Peak price delta for iceberg orders. * The ppd of buy orders must be smaller or equal than zero. * The ppd of sell orders must be greater or equal than zero. If it is omitted the system will assume a value of `0.00`.  # noqa: E501

        :param ppd: The ppd of this OrderModify.  # noqa: E501
        :type ppd: float
        """

        self._ppd = ppd

    @property
    def expected_net_pos(self):
        """Gets the expected_net_pos of this OrderModify.  # noqa: E501

        The expected net_pos (see portfolio information for more details). Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :return: The expected_net_pos of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_net_pos

    @expected_net_pos.setter
    def expected_net_pos(self, expected_net_pos):
        """Sets the expected_net_pos of this OrderModify.

        The expected net_pos (see portfolio information for more details). Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :param expected_net_pos: The expected_net_pos of this OrderModify.  # noqa: E501
        :type expected_net_pos: float
        """

        self._expected_net_pos = expected_net_pos

    @property
    def expected_min_net_pos(self):
        """Gets the expected_min_net_pos of this OrderModify.  # noqa: E501

        The expected net_pos (see portfolio information for more details) plus the absolute quantity of all standing SELL-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :return: The expected_min_net_pos of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_min_net_pos

    @expected_min_net_pos.setter
    def expected_min_net_pos(self, expected_min_net_pos):
        """Sets the expected_min_net_pos of this OrderModify.

        The expected net_pos (see portfolio information for more details) plus the absolute quantity of all standing SELL-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :param expected_min_net_pos: The expected_min_net_pos of this OrderModify.  # noqa: E501
        :type expected_min_net_pos: float
        """

        self._expected_min_net_pos = expected_min_net_pos

    @property
    def expected_max_net_pos(self):
        """Gets the expected_max_net_pos of this OrderModify.  # noqa: E501

        The expected net_pos (see portfolio information for more details) plus the absolute quantity of all standing BUY-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :return: The expected_max_net_pos of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_max_net_pos

    @expected_max_net_pos.setter
    def expected_max_net_pos(self, expected_max_net_pos):
        """Sets the expected_max_net_pos of this OrderModify.

        The expected net_pos (see portfolio information for more details) plus the absolute quantity of all standing BUY-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :param expected_max_net_pos: The expected_max_net_pos of this OrderModify.  # noqa: E501
        :type expected_max_net_pos: float
        """

        self._expected_max_net_pos = expected_max_net_pos

    @property
    def expected_min_net_pos_exposure(self):
        """Gets the expected_min_net_pos_exposure of this OrderModify.  # noqa: E501

        The expected min_net_pos_exposure. See portfolio information min_net_pos_exposure for more details.  # noqa: E501

        :return: The expected_min_net_pos_exposure of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_min_net_pos_exposure

    @expected_min_net_pos_exposure.setter
    def expected_min_net_pos_exposure(self, expected_min_net_pos_exposure):
        """Sets the expected_min_net_pos_exposure of this OrderModify.

        The expected min_net_pos_exposure. See portfolio information min_net_pos_exposure for more details.  # noqa: E501

        :param expected_min_net_pos_exposure: The expected_min_net_pos_exposure of this OrderModify.  # noqa: E501
        :type expected_min_net_pos_exposure: float
        """

        self._expected_min_net_pos_exposure = expected_min_net_pos_exposure

    @property
    def expected_max_net_pos_exposure(self):
        """Gets the expected_max_net_pos_exposure of this OrderModify.  # noqa: E501

        The expected max_net_pos_exposure. See portfolio information max_net_pos_exposure for more details.  # noqa: E501

        :return: The expected_max_net_pos_exposure of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_max_net_pos_exposure

    @expected_max_net_pos_exposure.setter
    def expected_max_net_pos_exposure(self, expected_max_net_pos_exposure):
        """Sets the expected_max_net_pos_exposure of this OrderModify.

        The expected max_net_pos_exposure. See portfolio information max_net_pos_exposure for more details.  # noqa: E501

        :param expected_max_net_pos_exposure: The expected_max_net_pos_exposure of this OrderModify.  # noqa: E501
        :type expected_max_net_pos_exposure: float
        """

        self._expected_max_net_pos_exposure = expected_max_net_pos_exposure

    @property
    def expected_abs_pos(self):
        """Gets the expected_abs_pos of this OrderModify.  # noqa: E501

        The expected abs_pos (see portfolio information for more details). Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :return: The expected_abs_pos of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_abs_pos

    @expected_abs_pos.setter
    def expected_abs_pos(self, expected_abs_pos):
        """Sets the expected_abs_pos of this OrderModify.

        The expected abs_pos (see portfolio information for more details). Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :param expected_abs_pos: The expected_abs_pos of this OrderModify.  # noqa: E501
        :type expected_abs_pos: float
        """

        self._expected_abs_pos = expected_abs_pos

    @property
    def expected_sell_abs_pos(self):
        """Gets the expected_sell_abs_pos of this OrderModify.  # noqa: E501

        The expected abs_pos (see portfolio information for more details) plus the absolute quantity of all standing SELL-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :return: The expected_sell_abs_pos of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_sell_abs_pos

    @expected_sell_abs_pos.setter
    def expected_sell_abs_pos(self, expected_sell_abs_pos):
        """Sets the expected_sell_abs_pos of this OrderModify.

        The expected abs_pos (see portfolio information for more details) plus the absolute quantity of all standing SELL-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :param expected_sell_abs_pos: The expected_sell_abs_pos of this OrderModify.  # noqa: E501
        :type expected_sell_abs_pos: float
        """

        self._expected_sell_abs_pos = expected_sell_abs_pos

    @property
    def expected_buy_abs_pos(self):
        """Gets the expected_buy_abs_pos of this OrderModify.  # noqa: E501

        The expected abs_pos (see portfolio information for more details) plus the absolute quantity of all standing BUY-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :return: The expected_buy_abs_pos of this OrderModify.  # noqa: E501
        :rtype: float
        """
        return self._expected_buy_abs_pos

    @expected_buy_abs_pos.setter
    def expected_buy_abs_pos(self, expected_buy_abs_pos):
        """Sets the expected_buy_abs_pos of this OrderModify.

        The expected abs_pos (see portfolio information for more details) plus the absolute quantity of all standing BUY-Orders for the respective contract. Optional safety feature to mitigate race conditions if client and backend data are not in sync. Do not use this field for block products.  # noqa: E501

        :param expected_buy_abs_pos: The expected_buy_abs_pos of this OrderModify.  # noqa: E501
        :type expected_buy_abs_pos: float
        """

        self._expected_buy_abs_pos = expected_buy_abs_pos

    @property
    def location(self):
        """Gets the location of this OrderModify.  # noqa: E501

        Location within the delivery area. Should be only set for products with locations enabled.  # noqa: E501

        :return: The location of this OrderModify.  # noqa: E501
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this OrderModify.

        Location within the delivery area. Should be only set for products with locations enabled.  # noqa: E501

        :param location: The location of this OrderModify.  # noqa: E501
        :type location: str
        """

        self._location = location

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrderModify):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrderModify):
            return True

        return self.to_dict() != other.to_dict()



from collections import Counter
from typing import Dict, List, Tuple, Union


def reconstruct_order_book(
    raw_order_book_revisions: List[Dict[str, Union[str, int, List[Dict[str, Union[str, float]]]]]], with_best_prices: bool = True
) -> Dict[Tuple[str, str], Dict[str, Dict[str, List[Dict[str, Union[str, float]]]]]]:
    """Reconstruct order book from raw order book revisions.

        :param raw_order_book_revisions: List of raw order book revision received from the PowerBot API.
        :param with_best_prices: Whether to include the best bid and ask prices in each order book snapshot.
                                 If True, the output will contain additional dictionary statistics, that contains
                                 `best_bid`, `best_ask` and their corresponding quantities.

    :return: Reconstructed order book.

    Example:
        Data retrieval using powerbot_client
        >>> import json
        >>> from datetime import datetime

        >>> from powerbot_client import ApiClient, Configuration, HistoricDataApi
        >>> from powerbot_client.utils import reconstruct_order_book

        >>> api_key = "YOUR_API_KEY"
        >>> host_url = "YOUR_HOST_URL"
        >>> client = ApiClient(Configuration(api_key={"api_key_security": api_key}, host=host_url))
        >>> hist_api = HistoricDataApi(client)
        >>> delivery_start = datetime.fromisoformat("2024-05-01T12:00Z")
        >>> delivery_end = datetime.fromisoformat("2024-05-01T13:00Z")
        >>> product = "XBID_Hour_Power"
        >>> delivery_area = "SomeDeliveryArea"
        >>> page_size = 10_000
        >>> raw_order_book_revisions = []
        >>> token = None
        >>> while True:
        >>>     page = hist_api.get_historic_orders_with_http_info(
        >>>         delivery_area=delivery_area,
        >>>         delivery_start=delivery_start,
        >>>         delivery_end=delivery_end,
        >>>         product=product,
        >>>         page_size=page_size,
        >>>         page_token=token,
        >>>         _preload_content=False,
        >>>     )
        >>>     data = json.loads(page.data)
        >>>     raw_order_book_revisions.extend(data["revisions"])
        >>>     token = data.get("next_page_token")
        >>>     if token is None:
        >>>         break

        Data retrieval using powerbot_asyncio_client
        >>> import asyncio
        >>> import json
        >>> from datetime import datetime
        >>> from typing import Callable

        >>> from powerbot_asyncio_client import ApiClient, Configuration, HistoricDataApi
        >>> from powerbot_asyncio_client.utils import reconstruct_order_book

        >>> async def retrieve_raw_order_book_revisions(historic_request, delivery_area, delivery_start, delivery_end, product, page_size):
        >>>     raw_order_book_revisions = []
        >>>     token = None
        >>>     while True:
        >>>         page = await historic_request(
        >>>             delivery_area=delivery_area,
        >>>             delivery_start=delivery_start,
        >>>             delivery_end=delivery_end,
        >>>             product=product,
        >>>             page_size=page_size,
        >>>             page_token=token,
        >>>             _preload_content=False,
        >>>         )
        >>>         data = await page.content.read()
        >>>         data = json.loads(data)
        >>>         raw_order_book_revisions.extend(data["revisions"])
        >>>         token = data.get("next_page_token")
        >>>         if token is None:
        >>>             break
        >>>     return raw_order_book_revisions
        >>>
        >>> raw_order_book_revisions = asyncio.run(
        >>>     retrieve_raw_order_book_revisions(
        >>>         hist_api.get_historic_orders_with_http_info,
        >>>         delivery_start=delivery_start,
        >>>         delivery_end=delivery_end,
        >>>         product=product,
        >>>         page_size=page_size,
        >>>     )
        >>> )
        >>> reconstructed_order_book = reconstruct_order_book(raw_order_book_revisions)

    """
    bid_snapshot = {}
    ask_snapshot = {}

    book = {}
    # raw_order_book_revisions are already correctly sorted
    for revision in raw_order_book_revisions:
        current_bid = revision["buy_orders"]
        current_ask = revision["sell_orders"]

        # Keep all extra bids/asks and also those with quantity > 0. The latter are needed to get the best_bid/best_ask of new orders
        extra_bid = {o["order_id"]: o for o in current_bid}
        extra_bid_qty_filt = {o["order_id"]: o for o in current_bid if o["quantity"] > 0}

        extra_ask = {o["order_id"]: o for o in current_ask}
        extra_ask_qty_filt = {o["order_id"]: o for o in current_ask if o["quantity"] > 0}

        # Get best_bid and best_ask of the new orders
        new_best_bid = sorted(extra_bid_qty_filt.values(), key=lambda x: x["price"], reverse=True)[0]["price"] if extra_bid_qty_filt else -10000
        new_best_ask = sorted(extra_ask_qty_filt.values(), key=lambda x: x["price"])[0]["price"] if extra_ask_qty_filt else 10000

        # Remove every order from the last snapshot if
        # (1) its order-id is present in the new orders (extra_bid, extra_ask)
        bid_snapshot = bid_snapshot | extra_bid
        ask_snapshot = ask_snapshot | extra_ask

        # (2) If the order has a worse price than the best opposing order of the new orders. Important to note:
        # If this happens, it means that the deletion of that order has not been persisted correctly. It is not clear when the actual deletion
        # of that order did happen, but this timestamp is the latest point where the order can for sure not be available anymore and needs to
        # be removed.
        # (3) Remove orders that have quantity 0
        bid_snapshot = {o_id: o for o_id, o in bid_snapshot.items() if o["quantity"] > 0 and o["price"] < new_best_ask}
        ask_snapshot = {o_id: o for o_id, o in ask_snapshot.items() if o["quantity"] > 0 and o["price"] > new_best_bid}

        book[(revision["time"], revision["revision_number"])] = {"ask": ask_snapshot, "bid": bid_snapshot}

        if with_best_prices:
            bid_counter = Counter()

            for order in bid_snapshot.values():
                bid_counter[order["price"]] += order["quantity"]

            best_bid_price, best_bid_quantity = max(bid_counter.items(), key=lambda x: x[0], default=(None, None))

            sell_counter = Counter()

            for order in ask_snapshot.values():
                sell_counter[order["price"]] += order["quantity"]

            best_ask_price, best_ask_quantity = min(sell_counter.items(), key=lambda x: x[0], default=(None, None))

            book[(revision["time"], revision["revision_number"])].update(
                {
                    "best_bid_price": best_bid_price,
                    "best_bid_quantity": best_bid_quantity,
                    "best_ask_price": best_ask_price,
                    "best_ask_quantity": best_ask_quantity,
                }
            )

    return book



# coding: utf-8

"""
    PowerBot - Webservice for algotrading

    # TERMS AND CONDITIONS The PowerBot system provides B2B services for trading at intraday power markets. By using the PowerBot service, each user agrees to the terms and conditions of this licence: 1. The user confirms that they are familiar with the exchanges trading system and all relevant rules, is professionally qualified and in possession of a trading license for the relevant exchange. 2. The user will comply with the exchanges market rules (e.g. [EPEX Spot Market Rules](https://www.epexspot.com/en/downloads#rules-fees-processes) or [Nord Pool Market Rules](https://www.nordpoolgroup.com/trading/Rules-and-regulations/)) and will not endanger the exchange system at any time with heavy load from trading algorithms or by other use. 3. The user is aware of limits imposed by the exchange. 4. The user is solely liable for actions resulting from the use of PowerBot.   # INTRODUCTION PowerBot is a web-based software service enabling algorithmic trading on intraday power exchanges such as EPEX, Nord Pool, HUPX, BSP Southpool, TGE, OPCOM or ETPA. The service is straightforward to integrate in an existing software environment and provides a variety of programming interfaces for development of individual trading algorithms and software tools. Besides enabling fully automated intraday trading, it can be used to create tools for human traders providing relevant information and trading opportunities or can be integrated in existing software tools. For further details see https://www.powerbot-trading.com  ## Knowledge Base In addition to this API guide, please find the documentation at https://docs.powerbot-trading.com - the password will be provided by the PowerBot team. If not, please reach out to us at support@powerbot-trading.com  ## Endpoints The PowerBot service is available at the following REST endpoints:  | Instance      | Base URL for REST Endpoints                                      | |---------------|------------------------------------------------------------------| | EPEX          | https://staging.powerbot-trading.com/playground/epex/v2/api      | | Nord Pool     | https://staging.powerbot-trading.com/playground/nordpool/v2/api  | | HUPX          | https://staging.powerbot-trading.com/playground/hupx/v2/api      | | BSP Southpool | https://staging.powerbot-trading.com/playground/southpool/v2/api | | TGE           | https://staging.powerbot-trading.com/playground/tge/v2/api       | | IBEX          | https://staging.powerbot-trading.com/playground/ibex/v2/api      | | CROPEX        | https://staging.powerbot-trading.com/playground/cropex/v2/api    | | OPCOM         | https://staging.powerbot-trading.com/playground/opcom/v2/api     | | ETPA          | https://staging.powerbot-trading.com/playground/etpa/v2/api      | | BRM           | https://staging.powerbot-trading.com/playground/brm/v2/api       |  Access to endpoints is secured via an API Key, which needs to be passed as an \"api_key\" header in each request.   Notes on API Keys:  * API keys are specific to Test, Staging or Production.  * API keys are generated by the system administrator and need to be requested.  ## How to generate API clients (libraries) This OpenAPI specification can be used to generate API clients (programming libraries) for a wide range of programming languages using tools like [OpenAPI Generator](https://openapi-generator.tech/). A detailed guide can be found in the [knowledge base](https://docs.powerbot-trading.com/articles/getting-started/generating-clients/).  ## PowerBot Python client For Python, a ready-made client is also available on PyPI and can be downloaded locally via:  ```shell   pip install powerbot-client ```  ## Errors The API uses standard HTTP status codes to indicate the success or failure of the API call. The body of the response will be in JSON format as follows:  ``` {   \"message\": \"... an error message ...\" } ```  ## Paging The API uses offset and limit parameters for paged operations. An X-Total-Count header is added to responses to indicate the total number of items in a paged response.  ## API Rate Limiting The API limits the number of concurrent calls to 50 - when that limit is reached, the client will receive 503 http status codes (service unavailable) with the following text:  ``` {   \"message\": \"API rate limit exceeded\" } ``` Clients should ensure that they stay within the limit for concurrent API calls.    ## Additional code samples Additional information and code samples demonstrating the use of the API can be found at in our [knowledge base](https://docs.powerbot-trading.com/docs/programmatic-access/)  # noqa: E501

    The version of the OpenAPI document: 2.21.0-rc1
    Contact: office@powerbot-trading.com
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from powerbot_asyncio_client.configuration import Configuration


class PortfolioInformation(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'portfolio_id': 'str',
        'currency': 'str',
        'net_cash_pos': 'float',
        'trades_spending': 'float',
        'trades_earnings': 'float',
        'orders_spending': 'float',
        'orders_earnings': 'float',
        'net_pos': 'float',
        'net_pos_aggregate': 'float',
        'net_pos_exposure': 'float',
        'max_net_pos_exposure': 'float',
        'min_net_pos_exposure': 'float',
        'max_net_pos_buy_with_orders': 'float',
        'min_net_pos_sell_with_orders': 'float',
        'abs_pos': 'float',
        'sell_abs_pos': 'float',
        'buy_abs_pos': 'float',
        'order_action_quota': 'float',
        'auction_positions': 'AuctionPositions'
    }

    attribute_map = {
        'portfolio_id': 'portfolio_id',
        'currency': 'currency',
        'net_cash_pos': 'net_cash_pos',
        'trades_spending': 'trades_spending',
        'trades_earnings': 'trades_earnings',
        'orders_spending': 'orders_spending',
        'orders_earnings': 'orders_earnings',
        'net_pos': 'net_pos',
        'net_pos_aggregate': 'net_pos_aggregate',
        'net_pos_exposure': 'net_pos_exposure',
        'max_net_pos_exposure': 'max_net_pos_exposure',
        'min_net_pos_exposure': 'min_net_pos_exposure',
        'max_net_pos_buy_with_orders': 'max_net_pos_buy_with_orders',
        'min_net_pos_sell_with_orders': 'min_net_pos_sell_with_orders',
        'abs_pos': 'abs_pos',
        'sell_abs_pos': 'sell_abs_pos',
        'buy_abs_pos': 'buy_abs_pos',
        'order_action_quota': 'order_action_quota',
        'auction_positions': 'auction_positions'
    }

    def __init__(self, portfolio_id=None, currency=None, net_cash_pos=None, trades_spending=None, trades_earnings=None, orders_spending=None, orders_earnings=None, net_pos=None, net_pos_aggregate=None, net_pos_exposure=None, max_net_pos_exposure=None, min_net_pos_exposure=None, max_net_pos_buy_with_orders=None, min_net_pos_sell_with_orders=None, abs_pos=None, sell_abs_pos=None, buy_abs_pos=None, order_action_quota=None, auction_positions=None, local_vars_configuration=None):  # noqa: E501
        """PortfolioInformation - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._portfolio_id = None
        self._currency = None
        self._net_cash_pos = None
        self._trades_spending = None
        self._trades_earnings = None
        self._orders_spending = None
        self._orders_earnings = None
        self._net_pos = None
        self._net_pos_aggregate = None
        self._net_pos_exposure = None
        self._max_net_pos_exposure = None
        self._min_net_pos_exposure = None
        self._max_net_pos_buy_with_orders = None
        self._min_net_pos_sell_with_orders = None
        self._abs_pos = None
        self._sell_abs_pos = None
        self._buy_abs_pos = None
        self._order_action_quota = None
        self._auction_positions = None
        self.discriminator = None

        if portfolio_id is not None:
            self.portfolio_id = portfolio_id
        if currency is not None:
            self.currency = currency
        if net_cash_pos is not None:
            self.net_cash_pos = net_cash_pos
        if trades_spending is not None:
            self.trades_spending = trades_spending
        if trades_earnings is not None:
            self.trades_earnings = trades_earnings
        if orders_spending is not None:
            self.orders_spending = orders_spending
        if orders_earnings is not None:
            self.orders_earnings = orders_earnings
        if net_pos is not None:
            self.net_pos = net_pos
        if net_pos_aggregate is not None:
            self.net_pos_aggregate = net_pos_aggregate
        if net_pos_exposure is not None:
            self.net_pos_exposure = net_pos_exposure
        if max_net_pos_exposure is not None:
            self.max_net_pos_exposure = max_net_pos_exposure
        if min_net_pos_exposure is not None:
            self.min_net_pos_exposure = min_net_pos_exposure
        if max_net_pos_buy_with_orders is not None:
            self.max_net_pos_buy_with_orders = max_net_pos_buy_with_orders
        if min_net_pos_sell_with_orders is not None:
            self.min_net_pos_sell_with_orders = min_net_pos_sell_with_orders
        if abs_pos is not None:
            self.abs_pos = abs_pos
        if sell_abs_pos is not None:
            self.sell_abs_pos = sell_abs_pos
        if buy_abs_pos is not None:
            self.buy_abs_pos = buy_abs_pos
        if order_action_quota is not None:
            self.order_action_quota = order_action_quota
        if auction_positions is not None:
            self.auction_positions = auction_positions

    @property
    def portfolio_id(self):
        """Gets the portfolio_id of this PortfolioInformation.  # noqa: E501


        :return: The portfolio_id of this PortfolioInformation.  # noqa: E501
        :rtype: str
        """
        return self._portfolio_id

    @portfolio_id.setter
    def portfolio_id(self, portfolio_id):
        """Sets the portfolio_id of this PortfolioInformation.


        :param portfolio_id: The portfolio_id of this PortfolioInformation.  # noqa: E501
        :type portfolio_id: str
        """

        self._portfolio_id = portfolio_id

    @property
    def currency(self):
        """Gets the currency of this PortfolioInformation.  # noqa: E501

        the currency used for the request context  # noqa: E501

        :return: The currency of this PortfolioInformation.  # noqa: E501
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this PortfolioInformation.

        the currency used for the request context  # noqa: E501

        :param currency: The currency of this PortfolioInformation.  # noqa: E501
        :type currency: str
        """

        self._currency = currency

    @property
    def net_cash_pos(self):
        """Gets the net_cash_pos of this PortfolioInformation.  # noqa: E501

        The portfolio related net cash position without standing orders. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). It is calculated by the sum of all trades increasing the cash-position minus the sum all trades decreasing the cash-position. Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The net_cash_pos of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._net_cash_pos

    @net_cash_pos.setter
    def net_cash_pos(self, net_cash_pos):
        """Sets the net_cash_pos of this PortfolioInformation.

        The portfolio related net cash position without standing orders. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). It is calculated by the sum of all trades increasing the cash-position minus the sum all trades decreasing the cash-position. Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param net_cash_pos: The net_cash_pos of this PortfolioInformation.  # noqa: E501
        :type net_cash_pos: float
        """

        self._net_cash_pos = net_cash_pos

    @property
    def trades_spending(self):
        """Gets the trades_spending of this PortfolioInformation.  # noqa: E501

        Sum of the costs of all trades for the given portfolio which are decreasing the cash position. Orders decreasing the cash position are BUY-Trades with a price greater than 0 and SELL-Trades with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The trades_spending of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._trades_spending

    @trades_spending.setter
    def trades_spending(self, trades_spending):
        """Sets the trades_spending of this PortfolioInformation.

        Sum of the costs of all trades for the given portfolio which are decreasing the cash position. Orders decreasing the cash position are BUY-Trades with a price greater than 0 and SELL-Trades with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param trades_spending: The trades_spending of this PortfolioInformation.  # noqa: E501
        :type trades_spending: float
        """

        self._trades_spending = trades_spending

    @property
    def trades_earnings(self):
        """Gets the trades_earnings of this PortfolioInformation.  # noqa: E501

        Sum of the costs of all trades for the given portfolio which are increasing the cash position. Orders increasing the cash position are SELL-Trades with a price greater than 0 and BUY-Trades with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The trades_earnings of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._trades_earnings

    @trades_earnings.setter
    def trades_earnings(self, trades_earnings):
        """Sets the trades_earnings of this PortfolioInformation.

        Sum of the costs of all trades for the given portfolio which are increasing the cash position. Orders increasing the cash position are SELL-Trades with a price greater than 0 and BUY-Trades with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param trades_earnings: The trades_earnings of this PortfolioInformation.  # noqa: E501
        :type trades_earnings: float
        """

        self._trades_earnings = trades_earnings

    @property
    def orders_spending(self):
        """Gets the orders_spending of this PortfolioInformation.  # noqa: E501

        The cash-position of all standing orders (excluding trades) for the given portfolio decreasing the net cash position. Orders decreasing the cash position are BUY-Orders with a price greater than 0 and SELL-Orders with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts. For an iceberg order the maximum spending will be calculated and used to determine if the order can be placed.  # noqa: E501

        :return: The orders_spending of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._orders_spending

    @orders_spending.setter
    def orders_spending(self, orders_spending):
        """Sets the orders_spending of this PortfolioInformation.

        The cash-position of all standing orders (excluding trades) for the given portfolio decreasing the net cash position. Orders decreasing the cash position are BUY-Orders with a price greater than 0 and SELL-Orders with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts. For an iceberg order the maximum spending will be calculated and used to determine if the order can be placed.  # noqa: E501

        :param orders_spending: The orders_spending of this PortfolioInformation.  # noqa: E501
        :type orders_spending: float
        """

        self._orders_spending = orders_spending

    @property
    def orders_earnings(self):
        """Gets the orders_earnings of this PortfolioInformation.  # noqa: E501

        The cash-position of all standing orders (excluding trades) for the given portfolio increasing the net cash position. Orders increasing the cash position are SELL-Orders with a price greater than 0 and BUY-Orders with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts. For an iceberg order the maximum earning will be calculated and used to determine if the order can be placed.  # noqa: E501

        :return: The orders_earnings of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._orders_earnings

    @orders_earnings.setter
    def orders_earnings(self, orders_earnings):
        """Sets the orders_earnings of this PortfolioInformation.

        The cash-position of all standing orders (excluding trades) for the given portfolio increasing the net cash position. Orders increasing the cash position are SELL-Orders with a price greater than 0 and BUY-Orders with a price less than 0. The scope is spanning across all exchanges on a portfolio-level for the date of the contract (converted to local time). Please note that the timeframe includes both XBID and LOCAL contracts. For an iceberg order the maximum earning will be calculated and used to determine if the order can be placed.  # noqa: E501

        :param orders_earnings: The orders_earnings of this PortfolioInformation.  # noqa: E501
        :type orders_earnings: float
        """

        self._orders_earnings = orders_earnings

    @property
    def net_pos(self):
        """Gets the net_pos of this PortfolioInformation.  # noqa: E501

        The net position (quantity of BUY-Trades minus SELL-Trades) without standing orders for the timeframe of the particular contract, portfolio and exchange. Contracts with a timeframe that falls into the timeframe of this particular contract are not included (e.g., quarterhourly contracts do not affect the net_pos of the hourly contract). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The net_pos of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._net_pos

    @net_pos.setter
    def net_pos(self, net_pos):
        """Sets the net_pos of this PortfolioInformation.

        The net position (quantity of BUY-Trades minus SELL-Trades) without standing orders for the timeframe of the particular contract, portfolio and exchange. Contracts with a timeframe that falls into the timeframe of this particular contract are not included (e.g., quarterhourly contracts do not affect the net_pos of the hourly contract). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param net_pos: The net_pos of this PortfolioInformation.  # noqa: E501
        :type net_pos: float
        """

        self._net_pos = net_pos

    @property
    def net_pos_aggregate(self):
        """Gets the net_pos_aggregate of this PortfolioInformation.  # noqa: E501

        The aggregated net position of all contracts which fully overlap with the delivery period of this contract (e.g., for quarterhourly contracts this will include the net_pos of the quarter-hourly, the half-hourly and the hourly contracts). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The net_pos_aggregate of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._net_pos_aggregate

    @net_pos_aggregate.setter
    def net_pos_aggregate(self, net_pos_aggregate):
        """Sets the net_pos_aggregate of this PortfolioInformation.

        The aggregated net position of all contracts which fully overlap with the delivery period of this contract (e.g., for quarterhourly contracts this will include the net_pos of the quarter-hourly, the half-hourly and the hourly contracts). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param net_pos_aggregate: The net_pos_aggregate of this PortfolioInformation.  # noqa: E501
        :type net_pos_aggregate: float
        """

        self._net_pos_aggregate = net_pos_aggregate

    @property
    def net_pos_exposure(self):
        """Gets the net_pos_exposure of this PortfolioInformation.  # noqa: E501

        The sum of all trade quantities (buy minus sell) which overlap with the delivery period of this contract.  # noqa: E501

        :return: The net_pos_exposure of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._net_pos_exposure

    @net_pos_exposure.setter
    def net_pos_exposure(self, net_pos_exposure):
        """Sets the net_pos_exposure of this PortfolioInformation.

        The sum of all trade quantities (buy minus sell) which overlap with the delivery period of this contract.  # noqa: E501

        :param net_pos_exposure: The net_pos_exposure of this PortfolioInformation.  # noqa: E501
        :type net_pos_exposure: float
        """

        self._net_pos_exposure = net_pos_exposure

    @property
    def max_net_pos_exposure(self):
        """Gets the max_net_pos_exposure of this PortfolioInformation.  # noqa: E501

        The highest net_pos_exposure within the smallest delivery period of this contract. max_net_pos_exposure = max(net_pos_exposure + quantity of all outstanding buy orders) within the smallest delivery period.  # noqa: E501

        :return: The max_net_pos_exposure of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._max_net_pos_exposure

    @max_net_pos_exposure.setter
    def max_net_pos_exposure(self, max_net_pos_exposure):
        """Sets the max_net_pos_exposure of this PortfolioInformation.

        The highest net_pos_exposure within the smallest delivery period of this contract. max_net_pos_exposure = max(net_pos_exposure + quantity of all outstanding buy orders) within the smallest delivery period.  # noqa: E501

        :param max_net_pos_exposure: The max_net_pos_exposure of this PortfolioInformation.  # noqa: E501
        :type max_net_pos_exposure: float
        """

        self._max_net_pos_exposure = max_net_pos_exposure

    @property
    def min_net_pos_exposure(self):
        """Gets the min_net_pos_exposure of this PortfolioInformation.  # noqa: E501

        The lowest net_pos_exposure within the smallest delivery period of this contract. min_net_pos_exposure = min(net_pos_exposure - quantity of all outstanding sell orders) within the smallest delivery period.  # noqa: E501

        :return: The min_net_pos_exposure of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._min_net_pos_exposure

    @min_net_pos_exposure.setter
    def min_net_pos_exposure(self, min_net_pos_exposure):
        """Sets the min_net_pos_exposure of this PortfolioInformation.

        The lowest net_pos_exposure within the smallest delivery period of this contract. min_net_pos_exposure = min(net_pos_exposure - quantity of all outstanding sell orders) within the smallest delivery period.  # noqa: E501

        :param min_net_pos_exposure: The min_net_pos_exposure of this PortfolioInformation.  # noqa: E501
        :type min_net_pos_exposure: float
        """

        self._min_net_pos_exposure = min_net_pos_exposure

    @property
    def max_net_pos_buy_with_orders(self):
        """Gets the max_net_pos_buy_with_orders of this PortfolioInformation.  # noqa: E501

        The net_pos, including standing BUY orders for the timeframe of the particular contract, portfolio and exchange. Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The max_net_pos_buy_with_orders of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._max_net_pos_buy_with_orders

    @max_net_pos_buy_with_orders.setter
    def max_net_pos_buy_with_orders(self, max_net_pos_buy_with_orders):
        """Sets the max_net_pos_buy_with_orders of this PortfolioInformation.

        The net_pos, including standing BUY orders for the timeframe of the particular contract, portfolio and exchange. Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param max_net_pos_buy_with_orders: The max_net_pos_buy_with_orders of this PortfolioInformation.  # noqa: E501
        :type max_net_pos_buy_with_orders: float
        """

        self._max_net_pos_buy_with_orders = max_net_pos_buy_with_orders

    @property
    def min_net_pos_sell_with_orders(self):
        """Gets the min_net_pos_sell_with_orders of this PortfolioInformation.  # noqa: E501

        The net_pos, including standing SELL orders for the timeframe of the particular contract, portfolio and exchange. Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The min_net_pos_sell_with_orders of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._min_net_pos_sell_with_orders

    @min_net_pos_sell_with_orders.setter
    def min_net_pos_sell_with_orders(self, min_net_pos_sell_with_orders):
        """Sets the min_net_pos_sell_with_orders of this PortfolioInformation.

        The net_pos, including standing SELL orders for the timeframe of the particular contract, portfolio and exchange. Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param min_net_pos_sell_with_orders: The min_net_pos_sell_with_orders of this PortfolioInformation.  # noqa: E501
        :type min_net_pos_sell_with_orders: float
        """

        self._min_net_pos_sell_with_orders = min_net_pos_sell_with_orders

    @property
    def abs_pos(self):
        """Gets the abs_pos of this PortfolioInformation.  # noqa: E501

        The absolute position (Sum of all traded quantities) without standing orders for the timeframe of the particular contract, delivery area, portfolio and exchange. Contracts with a timeframe that falls into the timeframe of this particular contract are not included (e.g., quarterhourly contracts do not affect the abs_pos of the hourly contract). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :return: The abs_pos of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._abs_pos

    @abs_pos.setter
    def abs_pos(self, abs_pos):
        """Sets the abs_pos of this PortfolioInformation.

        The absolute position (Sum of all traded quantities) without standing orders for the timeframe of the particular contract, delivery area, portfolio and exchange. Contracts with a timeframe that falls into the timeframe of this particular contract are not included (e.g., quarterhourly contracts do not affect the abs_pos of the hourly contract). Please note that the timeframe includes both XBID and LOCAL contracts.  # noqa: E501

        :param abs_pos: The abs_pos of this PortfolioInformation.  # noqa: E501
        :type abs_pos: float
        """

        self._abs_pos = abs_pos

    @property
    def sell_abs_pos(self):
        """Gets the sell_abs_pos of this PortfolioInformation.  # noqa: E501

        The abs_pos plus the absolute quantity of all standing SELL-Orders for the respective contract.  # noqa: E501

        :return: The sell_abs_pos of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._sell_abs_pos

    @sell_abs_pos.setter
    def sell_abs_pos(self, sell_abs_pos):
        """Sets the sell_abs_pos of this PortfolioInformation.

        The abs_pos plus the absolute quantity of all standing SELL-Orders for the respective contract.  # noqa: E501

        :param sell_abs_pos: The sell_abs_pos of this PortfolioInformation.  # noqa: E501
        :type sell_abs_pos: float
        """

        self._sell_abs_pos = sell_abs_pos

    @property
    def buy_abs_pos(self):
        """Gets the buy_abs_pos of this PortfolioInformation.  # noqa: E501

        The abs_pos plus the absolute quantity of all standing BUY-Orders for the respective contract.  # noqa: E501

        :return: The buy_abs_pos of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._buy_abs_pos

    @buy_abs_pos.setter
    def buy_abs_pos(self, buy_abs_pos):
        """Sets the buy_abs_pos of this PortfolioInformation.

        The abs_pos plus the absolute quantity of all standing BUY-Orders for the respective contract.  # noqa: E501

        :param buy_abs_pos: The buy_abs_pos of this PortfolioInformation.  # noqa: E501
        :type buy_abs_pos: float
        """

        self._buy_abs_pos = buy_abs_pos

    @property
    def order_action_quota(self):
        """Gets the order_action_quota of this PortfolioInformation.  # noqa: E501

        **Only supported for EPEX, Nord Pool, IBEX and CROPEX**  The percentage (0.0 - 1.0) of consumed order entry/modification quota calculated depending on the underlying exchange.  This value is calculated based on the following formula:  `max{short_usage/short_limit, long_usage/long_limit}`  # noqa: E501

        :return: The order_action_quota of this PortfolioInformation.  # noqa: E501
        :rtype: float
        """
        return self._order_action_quota

    @order_action_quota.setter
    def order_action_quota(self, order_action_quota):
        """Sets the order_action_quota of this PortfolioInformation.

        **Only supported for EPEX, Nord Pool, IBEX and CROPEX**  The percentage (0.0 - 1.0) of consumed order entry/modification quota calculated depending on the underlying exchange.  This value is calculated based on the following formula:  `max{short_usage/short_limit, long_usage/long_limit}`  # noqa: E501

        :param order_action_quota: The order_action_quota of this PortfolioInformation.  # noqa: E501
        :type order_action_quota: float
        """

        self._order_action_quota = order_action_quota

    @property
    def auction_positions(self):
        """Gets the auction_positions of this PortfolioInformation.  # noqa: E501


        :return: The auction_positions of this PortfolioInformation.  # noqa: E501
        :rtype: AuctionPositions
        """
        return self._auction_positions

    @auction_positions.setter
    def auction_positions(self, auction_positions):
        """Sets the auction_positions of this PortfolioInformation.


        :param auction_positions: The auction_positions of this PortfolioInformation.  # noqa: E501
        :type auction_positions: AuctionPositions
        """

        self._auction_positions = auction_positions

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PortfolioInformation):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PortfolioInformation):
            return True

        return self.to_dict() != other.to_dict()
